/*
   This file is part of the Dvegas library for adaptive Monte Carlo
   integration inspired by Lepage's VEGAS algorithm.
   Copyright (C) 2000-2002, 2012 Nikolas Kauer, Martin Wiebusch

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


// http://sources.redhat.com/gsl/ref/gsl-ref_17.html#SEC253

#include <vector>
using std::vector;

#include "rng.h"
#include "gsl/gsl_rng.h"
#include <cassert>

class GSLRandomNumberGenerator : public RandomNumberGenerator
{
public:
  GSLRandomNumberGenerator(gsl_rng *const r, const long int seed);
  ~GSLRandomNumberGenerator();
  
  const double uniformInZeroOne();
  void setSeedAndLuxuryLevel(const long int seed, const int dummy);
  void varySeedToGetIndependentRandomNumberStreams(const long int unique);
private:
  long int _seed;
  gsl_rng* _gslRandomNumberGenerator;
private:     // prevent copy construction and assignment
  GSLRandomNumberGenerator(const GSLRandomNumberGenerator&);
  GSLRandomNumberGenerator& operator=(const GSLRandomNumberGenerator&);  
};

// --------------------------------------------------------------

RandomNumberGenerator *const randomNumberGenerator = new GSLRandomNumberGenerator(gsl_rng_alloc(gsl_rng_ranlxs1), 19780503);

// --------------------------------------------------------------

GSLRandomNumberGenerator::GSLRandomNumberGenerator(gsl_rng *const r, const long int seed)
  : _seed(0), _gslRandomNumberGenerator(r)
{
  setSeedAndLuxuryLevel(seed, 0);
}

GSLRandomNumberGenerator::~GSLRandomNumberGenerator()
{
  gsl_rng_free(_gslRandomNumberGenerator);
}

const double GSLRandomNumberGenerator::uniformInZeroOne()
{
  return gsl_rng_uniform_pos(_gslRandomNumberGenerator);
}

void GSLRandomNumberGenerator::setSeedAndLuxuryLevel(const long int seed, const int dummy)
{
  assert(seed >= 0);
  _seed = seed;
  gsl_rng_set(_gslRandomNumberGenerator, seed);
}

void GSLRandomNumberGenerator::varySeedToGetIndependentRandomNumberStreams(const long int unique)
{
  setSeedAndLuxuryLevel(_seed + unique, 0);
}
